import string, mmap, os
from utils import *
import kjbuckets
from searching import loop_in_C
import dictzip

class EmptyDb:
    def __init__(self, name):
        self.info = "empty database"
        self.name = name
        
    def match(self, strat, word):
        return []

    def define(self, word):
        return []

class FullDb:
    def __init__(self, name):
        self.info = "full database"
        self.name = name
        
    def match(self, strat, word):
        return [ (self.name, "one"), (self.name, "two") ]

    def define(self, word):
        return [ (self.name, "bla bla blablabla") ]


def dbargs(name, data=None, inx=None, info=None):
    """return tuple of: database name, database info, file object for data, mmap object for index
    """
    if not info:
        info = name
    if not data:
        data = name+".dict"
    sep = os.sep
    sep1 = os.altsep or os.sep
    if not( (sep in data) or (sep1 in data) ):
        data = "/usr/share/dictd/%s" % data
    if not inx:
        inx = name+".pyindex"
    if not( (sep in inx) or (sep1 in inx) ):
        inx = "/usr/share/dictd/%s" % inx
    try:
        datafo = open(data)
    except IOError:
        datafo = dictzip.DictzipFile(data+".dz")
    indexfo = open(inx)
    indexfo.seek(0, 2)
    indexlen = indexfo.tell()
    inx = mmap.mmap(indexfo.fileno(), indexlen, prot=mmap.PROT_READ)
    return name, info, datafo, inx


class FileDb:
    """Entries in index are in UTF8, sorted byte-after-byte
    """
    def __init__(self, **args):
        self.name, self.info, self.datafo, self.index = dbargs(**args)
        self.positions = []
        self.index.seek(0)
        while 1:
            p = self.index.tell()
            l = self.index.readline()
            if not l:
                break
            self.positions.append(p)

    def transformentry(self, s):
        # transforms read entry into plain text
        # or, in the future, into mime/html/sgml/whatever
        # to be overriden
        return s

    def readentry(self, arg):
        entry, st, ln = arg
        self.datafo.seek(st)
        r = self.transformentry(self.datafo.read(ln))
        return self.name, r


    def define(self, word):
        r = []
        poss = loop_in_C(self.index, self.positions, word, 0, 0)
        for i in poss:
            self.index.seek(self.positions[i])
            l = string.rstrip(self.index.readline())
            entry, st, ln = string.split(l, TAB)
            st, ln = b64dec(st), b64dec(ln)
            r.append( (entry,st,ln) )
        r = map(self.readentry, r)
        return r

                
    def match(self, strategy, word):
        r = []
        if strategy=='.':
            strategy = 'exact'
        strategy = strategies[strategy][0]
        # mmap pointer, list of index entries, word, strat, max nr
        r1 = loop_in_C(self.index, self.positions, word, strategy, 20)
        print `r1`
        for i in r1:
            self.index.seek(self.positions[i])
            l = self.index.readline()
            entry, st, ln = string.split(l, TAB, 2)
            st, ln = b64dec(st), b64dec(ln)
            r.append( (entry, st, ln) )
        for i in range(len(r)):
            r[i] = self.name, r[i][0]
        r = kjbuckets.kjSet(r).items()
        return r

        
    def __del__(self):
        self.datafo.close()
        self.index.close()



class FileDbDict(FileDb):
    """Entries in index are in UTF8, sorted byte-after-byte
       Database file is raw dict file (with %h, %d)
    """

    def transformentry(self, s):
        # transforms read entry into plain text
        rs = string.split(s, '\n')
        reply = []
        i = 0
        while 1:
            srs = string.lstrip(rs[i])
            if srs[:2] == '%h':
                reply.append(srs[2:])
                i+=1
            else:
                break
        if string.strip(rs[i])<>'%d':
            return 'wrong entry, please check'
        i+=1
        reply.append("-----") # separating entry and body
        reply.extend(rs[i:])
        return string.join(reply, '\n')
     